@@ -30,7 +30,7 @@ warnings.simplefilter("ignore")
 
 requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
 
-logging.basicConfig(format='%(asctime)s : %(message)s', filename='dyagram.log')
+logging.basicConfig(format='%(asctime)s : %(message)s', filename='dyagram.log', filemode='a')
 
 
 class Dyagram:
@@ -162,11 +162,11 @@ class Dyagram:
                     self.log.info(f"Pulled device {device} from Queue")
                     self.topology['devices'].append({'hostname': "", 'inventory_ip': device, 'layer2': {}, 'routes': [], 'dynamic_routing_neighbors': {}})
 
-                    future_lldp = executor.submit(self.__discover_lldp_neighbors, device)
+                    executor.submit(self.__discover_lldp_neighbors, device)
                     self.log.info(f"DEVICE: {device} - Submitted for discover_lldp_neighbors")
-                    future_routes = executor.submit(self.discover_routes, device)
+                    executor.submit(self.discover_routes, device)
                     self.log.info(f"DEVICE: {device} - Submitted for discover_routes")
-                    future_routing_neighbors = executor.submit(self.discover_dynamic_routing_neighbors, device)
+                    executor.submit(self.discover_dynamic_routing_neighbors, device)
                     self.log.info(f"DEVICE: {device} - Submitted for discover_dynamic_routing_neighbors")
 
                     self._devices_queried.append(device)
@@ -182,7 +182,8 @@ class Dyagram:
 
             #sort topology by hostname to compare to previous
 
-            self.topology['devices'] = sorted(self.topology['devices'], key=lambda d: d['hostname'])
+            #self.topology['devices'] = sorted(self.topology['devices'], key=lambda d: d['hostname'])
+            self.sort_topology() # sort topology to easily compare
 
             if not self.state_exists:
                 self.export_state()
@@ -195,7 +196,7 @@ class Dyagram:
                 file = open(rf"{self.site}/state.json", 'r')
                 state = json.load(file)
                 try:
-                    diffs = self.get_state_diff(state, self.topology) # state is the state file on record and topology is current state
+                    diffs = self.get_state_diff(state, self.topology,log=self.log) # state is the state file on record and topology is current state
                 except Exception:
                     tb = self.get_traceback()
                     self.log.info(f"EXCEPTION THROWN GETTING DIFFS: {tb}")
@@ -249,6 +250,18 @@ class Dyagram:
             self.changes_in_state = True
         file.close()
 
+    def sort_topology(self):
+        self.topology['devices'] = sorted(self.topology['devices'], key=lambda e: e['hostname'])
+        for d in self.topology['devices']:
+            for protocol in d['dynamic_routing_neighbors']:
+                d['dynamic_routing_neighbors'][protocol] = sorted(d['dynamic_routing_neighbors'][protocol])
+
+            d['routes'] = sorted(d['routes'], key=lambda e: e['network'])
+            d['layer2']['chassis_ids'] = sorted(d['layer2']['chassis_ids'])
+            d['layer2']['neighbors'] = sorted(d['layer2']['neighbors'], key=lambda e: e['chassis_id'])
+
+
+
 
     def discover_routes(self, device):
 
@@ -335,17 +348,17 @@ class Dyagram:
             #     dev = ConnectHandler(**netmiko_args)
             self.log.info(f"DEVICE: {device} - SSH : CREATING NETMIKO OBJ : discover_routes_ssh")
             tries = 0
-            while tries < 10: #FIXME: THIS SHOULD TRY 10 TIMES THEN RETURN UNABLE TO RUN DYAGRAM
+            while tries < 5: #FIXME: THIS SHOULD TRY 10 TIMES THEN RETURN UNABLE TO RUN DYAGRAM
                 try:
                     dev = ConnectHandler(**netmiko_args)
                     dev.enable()
-                    tries = 10  # break
+                    tries = 5  # break
                 except:
                     self.log.info(
                         f"DEVICE: {device} - SSH : FAILED TO CREATE NETMIKO OBJ, TRY AGAIN: discover_routes_ssh")
                     time.sleep(1)
                     tries += 1
-                    if tries == 10:
+                    if tries == 5:
                         self.log.info(f"DEVICE: {device} Unable to connect. Please, resolve and re-run Dyagram Discover.") #  CREATE FUNCTION THAT STOPS PROGRAM AND PRINTS SCREEN
 
 
@@ -373,16 +386,17 @@ class Dyagram:
             self.log.info(f"DEVICE: {device} EXCEPTION THROWN IN discover_routes_ssh: {tb}")
 
     @staticmethod
-    def get_state_diff(previous_state, current_state):
-
+    def get_state_diff(previous_state, current_state, log=None):
+        log.info(f"STARTING GET_STATE_DIFF")
         device_diffs = {}
 
         elem_changed = {
             "routes": []}  # this keeps up with "changes" when converting to removed/add for routes,
                            # this is due to deeper dictionary items
-
+        log.info(f"BEFORE DeepDiff")
         diff_resp = DeepDiff(previous_state, current_state, ignore_order=True)
-        print(diff_resp)
+        log.info(f"AFTER DeepDiff")
+        log.info(f"diff_resp = {diff_resp}")
         if not diff_resp:
             return None
 
@@ -492,7 +506,7 @@ class Dyagram:
 
                                     f"{current_state['devices'][device_elem][category][route_elem]['nexthop_ip']}")
 
-
+        log.info(f"END OF GET_STATE_DIFF")
         return device_diffs
 
     def get_device_type(self, device):
@@ -613,17 +627,18 @@ class Dyagram:
 
 
             tries = 0
-            while tries < 10:  # FIXME: THIS SHOULD TRY 10 TIMES THEN RETURN UNABLE TO RUN DYAGRAM
+            while tries < 5:  # FIXME: THIS SHOULD TRY 10 TIMES THEN RETURN UNABLE TO RUN DYAGRAM
                 try:
                     dev = ConnectHandler(**nm_args)
                     dev.enable()
-                    tries = 10  # break
+                    tries = 5  # break
                 except:
+                    tb = self.get_traceback()
                     self.log.info(
-                        f"DEVICE: {device} - SSH : FAILED TO CREATE NETMIKO OBJ, TRY AGAIN: discover_routes_ssh")
+                        f"DEVICE: {device} - SSH : FAILED TO CREATE NETMIKO OBJ, TRY AGAIN: discover_routes_ssh : TB = {tb}")
                     time.sleep(1)
                     tries += 1
-                    if tries == 10:
+                    if tries == 5:
                         self.log.info(f"DEVICE: {device} Unable to connect. Please, resolve and re-run Dyagram Discover.") #  CREATE FUNCTION THAT STOPS PROGRAM AND PRINTS SCREEN
 
             self.log.info(f"DEVICE: {device} CREATED NETMIKO OBJ : discover_eigrp_neighbors_ssh")
@@ -736,15 +751,15 @@ class Dyagram:
             self.log.info(f"DEVICE: {device} - SSH : CREATING NETMIKO OBJ : _discover_lldp_neighbors_by_ssh")
 
             tries = 0
-            while tries < 10: # FIXME: THIS SHOULD TRY 10 TIMES THEN RETURN UNABLE TO RUN DYAGRAM
+            while tries < 5: # FIXME: THIS SHOULD TRY 10 TIMES THEN RETURN UNABLE TO RUN DYAGRAM
                 try:
                     dev = ConnectHandler(**netmiko_args)
-                    tries = 10 # break
+                    tries = 5 # break
                 except:
                     self.log.info(f"DEVICE: {device} - SSH : FAILED TO CREATE NETMIKO OBJ, TRY AGAIN: _discover_lldp_neighbors_by_ssh")
                     time.sleep(1)
                     tries += 1
-                    if tries == 10:
+                    if tries == 5:
                         self.log.info(f"DEVICE: {device} Unable to connect. Please, resolve and re-run Dyagram Discover.") #  CREATE FUNCTION THAT STOPS PROGRAM AND PRINTS SCREEN
 
 
